#! /bin/tcsh -f
#
#	check if this unit cell occurs in the PDB
#
#
set reduced_cells_list = /home/jamesh/pdb/snapshot/reduced_cells.txt
set flip_map           = /home/jamesh/Develop/flipmap.txt

set path = ( $path /home/jamesh/Develop/ )

set tempfile = tempfile
set logfile  = debug.log

    set mtzfile = ./mtzfile.mtz
    if(-e "$1") then
	set mtzfile = "$1"
        echo "using $mtzfile"
    endif

    if($#argv >= 6) then
	set test = `echo $* | awk '$1+0>0 && $2+0>0 && $3+0>0 && $4+0>0 && $5+0>0 && $6+0>0'`
	if("$test" != "") then
	    set JUST_CELL
	    set CELL = `echo $* | awk '{print $1,$2,$3,$4,$5,$6}'`
	    set SG   = `echo $* | awk '$7~/^[PCIFHR]/{print $7} $1~/^[PCIFHR]/{print $1}'`
	    if("$SG" == "") set SG = P1
	    goto just_cell
	endif
    endif

    if(! -e rigid_body.com && -e `dirname $0`/rigid_body_checkcell.com) then
	cp `dirname $0`/rigid_body_checkcell.com rigid_body.com
    endif


    #get variables from mtz file
    echo "go" | mtzdump hklin $mtzfile | tee ${tempfile}mtzdump |\
    awk '/OVERALL FILE STATISTICS/,/No. of reflections used/' |\
    awk 'NF>10 && $(NF-1) ~ /[FQPWADI]/' |\
    cat >! ${tempfile}mtzdmp

    # use completeness, or F/sigF to pick default F
    cat ${tempfile}mtzdmp |\
    awk '$(NF-1) == "F"{F=$NF; meanF=$8; reso=$(NF-2); comp=substr($0,32)+0; \
      getline; S=$NF; if($8) meanF /= $8; print F, S, reso, comp, meanF;}' |\
    sort -k3n,4 -k4nr,5 -k5nr >! ${tempfile}F

    # pick F with best resolution, or F/sigma
    set F    = `head -1 ${tempfile}F`
    if($#F > 2) then
	set SIGF = $F[2]
	set F    = $F[1]
    endif



    set mtzSGnum = `awk '/Space group/{print $NF+0}'  ${tempfile}mtzdump`
    set mtzCELL  = `awk '/Cell Dimensions/{getline;getline;print}'  ${tempfile}mtzdump`
    set mtzSG   = `awk -v SGnum=$mtzSGnum '$1 == SGnum && $1 < 500 {print $4}' $CLIBD/symop.lib | head -1`
    set mtzPG   = `awk -v SGnum=$mtzSGnum '$1 == SGnum && $1 < 500 {print $5}' $CLIBD/symop.lib | head -1`

    set CELL = ( $mtzCELL )
    set SG = $mtzSG
just_cell:
    echo "CELL: $CELL"
    set latt  = `echo $SG | awk '{print $1}'`
    set centering = `echo $SG | awk '/^[ABCIF]/{c=substr($1,1,1);print c"CENTERED"}'`

    set test = `echo $latt $CELL | awk '{print ( /^[RH]/ && $5+0==90 && $6+0==90 && $7+0==120 )}'`
    if($test) then
	# get the rhombohedral cell - what you would get in indexing -
	set Hcell = ( $CELL )
 	echo "$Hcell" |\
	awk '{aH=$1;cH=$3;aR=sqrt(3*aH*aH+cH*cH)/3;\
	      sina2=3/2/sqrt(3+(cH/aH)^2);\
	      cosa2=sqrt(1-sina2^2);\
	      alphaR=atan2(sina2,cosa2)*2;\
	      alphaR=alphaR*90/atan2(1,0);\
	   print aR,aR,aR,alphaR,alphaR,alphaR}' |\
	cat >! Rcell.txt
	set CELL = `cat Rcell.txt`
	set reindex = "reindex hkl 2h/3+k/3+l/3,-h/3+k/3+l/3,-h/3-2k/3+l/3"
	set reindex_back = "reindex hkl h-k,k-l,h+k+l"
    else
	set reindex = ""
	set reindex_back = ""
    endif


    tracer.com $SG $CELL P1 | tee reindexing_choices.txt


    set reduced_cell = `awk '/conventional/{print $1,$2,$3,$4,$5,$6}' reindexing_choices.txt`

    echo "$reduced_cell"

    echo "SEARCH $reduced_cell" |\
    cat - $reduced_cells_list |\
    awk '/SEARCH/{A0=$2;B0=$3;C0=$4;alpha0=$5;beta0=$6;gamma0=$7;next}\
         {id=$1;A=$2;B=$3;C=$4;alpha=$5;beta=$6;gamma=$7;\
	  dele=sqrt((A-A0)^2 + (B-B0)^2 + (C-C0)^2);\
	  dela=sqrt((alpha-alpha0)^2 + (beta-beta0)^2 + (gamma-gamma0)^2);}\
         dele<30 && dela<30{print dele+dela,dele,dela,id,A0,B0,C0,alpha0,beta0,gamma0,A,B,C,alpha,beta,gamma;}' |\
    awk '$1!="nan"' |\
    sort --key=1,2g |\
    awk '{$1="";print}' |\
    head -300 |\
    tee hits.txt |\
    tac |\
    awk '{dele=$1;dela=$2;id=$3;A0=$4;B0=$5;C0=$6;alpha0=$7;beta0=$8;gamma0=$9;\
	                        A=$10;B=$11;C=$12;alpha=$13;beta=$14;gamma=$15;\
		print dele,dela;\
 		printf "%4s %6.1f %6.1f %6.1f %3.0f %3.0f %3.0f\n","",A0,B0,C0,alpha0,beta0,gamma0;\
                printf "%4s %6.1f %6.1f %6.1f %3.0f %3.0f %3.0f\n",id,A,B,C,alpha,beta,gamma}'

    if($?JUST_CELL) exit


    # get reindexing command to put things on the reduced cell

    set cellchoice = `awk '/conventional/{print $7}' reindexing_choices.txt`

    if("$reindex" == "") then
        set reindex = `awk -v choice="$cellchoice" '/cell/ && $3=="(1)" && $5==choice{getline;print}' reindexing_choices.txt`
    endif

    echo "reindexing by: $reindex and expanding data to get P1"
    cad hklin1 $mtzfile hklout expanded.mtz << EOF >> $logfile
labin file 1 E1=$F E2=$SIGF
outlim SPACE P1
EOF
    cad hklin1 expanded.mtz hklout P1.mtz << EOF >> $logfile
labin file 1 all
SYMM P1
EOF
    echo "$reindex" | reindex hklin P1.mtz hklout reindexed.mtz >> $logfile
    cad hklin1 reindexed.mtz hklout P1.mtz << EOF >> $logfile
labin file 1 all
SYMM P1
EOF

    # now do a sigma cutoff
    rm -f bright.mtz
    sftools << EOF >& $logfile
read P1.mtz
CALC COL SNR = COL $F COL $SIGF /
SELECT COL SNR > 6
write bright.mtz col $F $SIGF
exit
EOF
    echo "labin file 1 all" | cad hklin1 bright.mtz hklout refme.P1.mtz > $logfile






    foreach id ( `awk '{print $3}' hits.txt` )
	egrep "^$id " rigid_sumary.log
	if(! $status) continue

	getpdb.com $id >! pdbfile.pdb
	set pdbSG = `awk '/^CRYST/{print substr($0,56,12)}' pdbfile.pdb | head -1`
	set SG = `awk -v pdbSG="$pdbSG" -F "[\047]" 'pdbSG==$2{print;exit}' ${CLIBD}/symop.lib | awk '{print $4}'`
	if("$SG" == "") then
	    set SG = `echo $pdbSG | awk '{gsub(" ","");print}'`
	    set SG = `awk -v SG=$SG '$4 == SG && $1 < 500 {print $4}' $CLIBD/symop.lib | head -1`
	endif
	if("$SG" == "") then
	    set SG = `echo $pdbSG | awk '{gsub(" ","");print}'`
	endif
	set SG = `echo $SG | awk '{gsub("R","H"); print}'`
	set pdbCELL = `awk '/^CRYST1/{print $2,$3,$4,$5,$6,$7}' pdbfile.pdb`


	set pdbPG = `awk -v SG=$SG '$4 == SG && $1 < 500 {print $5}' $CLIBD/symop.lib | head -1`
	if("$pdbPG" != "$mtzPG") then
	    echo "wrong PG: $id $SG $CELL"
#	    #continue
	endif
	awk '/^ATOM/ || /^HETATM/ && (/HOH/ || /WAT/ || /OH2/)' pdbfile.pdb >! wet_atoms.pdb
	set wet_atoms = `cat wet_atoms.pdb | wc -l`
	echo "$wet_atoms atoms + water"

	egrep "^ATOM" pdbfile.pdb >! atoms.pdb
	set atoms = `cat atoms.pdb | wc -l`
	echo "$atoms atoms"

	# label everything as one chain
        cat atoms.pdb |\
        awk '/^ATOM/{Resnum = substr($0, 23, 4)+0}\
              Resnum!=lastResnum{lastResnum=Resnum;++res}\
            ! /^ATOM/{print} \
             /^ATOM/{print substr($0,1,20)" A" sprintf("%4d",res) substr($0,27)}' |\
        cat >! onechain.pdb



	set flips = `awk -v SG=$SG '$4==SG{gsub("h","x");gsub("k","y");gsub("l","z");for(i=5;i<=NF;++i)print $i}' $flip_map`
	set flips = `awk -v SG=$SG '$4==SG{for(i=5;i<=NF;++i)print $i}' $flip_map`
echo "GOTHERE $SG $flips"
        foreach flip ( $flips )

	    echo "trying: $id  $SG  $pdbCELL $flip"
	    egrep "^HEAD|^AUTH|^TITL" pdbfile.pdb

	    set test = `echo $SG $pdbCELL | awk '{print ( /^[RH]/ && $5+0==90 && $6+0==90 && $7+0==120 )}'`
	    if("$test" == "1") then
		# reindex back to hexagonal setting
		echo "reindex hkl h-k,k-l,h+k+l" |\
		reindex hklin refme.P1.mtz hklout refme.start.mtz >! $logfile
	    else
		cp refme.P1.mtz refme.start.mtz
		echo "" >! $logfile
	    endif

	    tracer.com $SG $pdbCELL P1 >! reindexing_choices.txt
	    set cellchoice = `awk '/conventional/{print $7}' reindexing_choices.txt`

	    set reindexhkl = `awk -v choice="$cellchoice" '/cell/ && $5=="(1)" && $3==choice{getline;print $3}' reindexing_choices.txt`
	    touch reindex_index.txt
	    set reindex_choice = `grep " $reindexhkl " reindex_index.txt | awk '{print $1;exit}'`
	    if("$reindex_choice" == "") then
		set choice = `awk '{print $1+1}' reindex_index.txt`
		if("$choice" == "") set choice = 1
		echo "$choice $reindexhkl " >> reindex_index.txt
	    endif
	    if(1 || ! -e refme.${reindex_choice}.mtz) then
		echo "reindexing: $reindexhkl"
		echo "reindex hkl $reindexhkl " | reindex hklin refme.start.mtz hklout refme.${reindex_choice}.mtz > $logfile
	    endif
	    if(1 || ! -e refme.${reindex_choice}.${SG}.mtz) then
		echo "cutting out $SG ASU from P1 expansion"
		cad hklin1 refme.${reindex_choice}.mtz hklout refme.${reindex_choice}.${SG}.mtz << EOF >  $logfile
		labin file 1 all
		outlim space $SG
		symm $SG
EOF
	    endif
	    if(1 || ! -e refme.${reindex_choice}.${SG}.${flip}.mtz) then
		echo "applying flip: $flip"
		echo "reindex hkl $flip" | reindex hklin refme.${reindex_choice}.${SG}.mtz hklout refme.${reindex_choice}.${SG}.${flip}.mtz >  $logfile
	    endif
	    echo "labin file 1 all" | cad hklin1 refme.${reindex_choice}.${SG}.${flip}.mtz hklout refme.mtz >  $logfile
	    echo "head" | mtzdump hklin refme.mtz >! tempfile
	    set mtzCELL = `awk '/Cell Dimensions/{getline;getline;print}' tempfile | tail -1`
	    echo "pdb cell: $pdbCELL"
	    echo "mtz cell: $mtzCELL"
	    

	    # make sure the cell is right...
	    echo "CELL $mtzCELL\nSYMGEN $flip" |\
	    pdbset xyzin onechain.pdb xyzout refme.pdb > /dev/null

	    echo "rigid body..."
	    ./rigid_body.com refme.pdb | tee rigid_body.log | grep "all R fac"
	    rm -f refme.pdb

	    set R = `awk '/Overall R factor/{print $NF}' rigid_body.log | tail -1`
	
	    echo "$id $R   $SG $pdbCELL $flip" | tee -a rigid_sumary.log
	end

    end

    cd ..
end

exit

cat << EOF >! rigid_body.com
#! /bin/csh -f
#	
#	auto-generated rigid-body script
#
set pdbin = "\$1"
if(! -e "\$pdbin") set pdbin = files/molrep.pdb

set mtzfile = ./refme.mtz
set tempfile = tempfile


#get variables from mtz file
echo "go" | mtzdump hklin \$mtzfile |\\
awk '/OVERALL FILE STATISTICS/,/No. of reflections used/' |\\
awk 'NF>10 && \$(NF-1) ~ /[FQPWADI]/' |\\
cat >! \${tempfile}mtzdmp

# use completeness, or F/sigF to pick default F
cat \${tempfile}mtzdmp |\\
awk '\$(NF-1) == "F"{F=\$NF; meanF=\$8; reso=\$(NF-2); comp=substr(\$0,32)+0; \\
      getline; S=\$NF; if(\$8) meanF /= \$8; print F, S, reso, comp, meanF;}' |\\
sort -k3n,4 -k4nr,5 -k5nr >! \${tempfile}F

# and extract all dataset types/labels
cat \${tempfile}mtzdmp |\\
awk 'NF>2{print \$(NF-1), \$NF, " "}' |\\
cat >! \${tempfile}cards

# pick F with best resolution, or F/sigma
set F    = \`head -1 \${tempfile}F\`
if(\$#F > 2) then
    set SIGF = \$F[2]
    set F    = \$F[1]
endif

refmac5 HKLIN \$mtzfile HKLOUT ./refmacout.mtz \\
XYZIN  \$pdbin XYZOUT rigid_body.pdb << EOF-refmac
LABIN FP=\$F SIGFP=\$SIGF
LABO FC=FC PHIC=PHIC    FWT=2FOFCWT PHWT=PH2FOFCWT -
                     DELFWT=FOFCWT  PHDELWT=PHFOFCWT
REFI TYPE RIGID
RIGID GROUP 1 A 1 to 7500
RIGID GROUP 1 B 1 to 7500
RIGID GROUP 1 C 1 to 7500
RIGID GROUP 1 D 1 to 7500
RIGID GROUP 1 E 1 to 7500
RIGID GROUP 1 F 1 to 7500
RIGID GROUP 1 G 1 to 7500
RIGID GROUP 1 H 1 to 7500
RIGID GROUP 1 I 1 to 7500
RIGID GROUP 1 J 1 to 7500
RIGID GROUP 1 K 1 to 7500
RIGID GROUP 1 L 1 to 7500
RIGID GROUP 1 M 1 to 7500
RIGID GROUP 1 N 1 to 7500
RIGID GROUP 1 O 1 to 7500
RIGID GROUP 1 P 1 to 7500
RIGID GROUP 1 Q 1 to 7500
RIGID GROUP 1 R 1 to 7500
RIGID GROUP 1 S 1 to 7500
RIGID GROUP 1 T 1 to 7500
RIGID GROUP 1 U 1 to 7500
RIGID GROUP 1 V 1 to 7500
RIGID GROUP 1 W 1 to 7500
RIGID GROUP 1 X 1 to 7500
RIGID GROUP 1 Y 1 to 7500
RIGID GROUP 1 Z 1 to 7500
RIGID NCYC 10

SOLVENT NO
REFI RESIDUAL MLKF METHOD CGRAD
REFI RESO 6 3.0
REFI BREF OVERALL
REFI PHASE SIGMAcalc
EOF-refmac

EOF
chmod a+x rigid_body.com





set besthit = `sort -n -k2 rigid_sumary.log | awk '{print $1;exit}'`
mkdir -p besthit
cd besthit
getpdb.com $besthit >! hit.pdb
source /programs/arp_warp_6.1/warprc
setup_warp.com ../mtzfile.mtz hit.pdb
~jamesh/Develop/warp.runme >&! warp1.log


exit

rm -f sequences.txt
foreach id ( `sort -n -k2 rigid_sumary.log | awk '$2+0>0{print $1}'` )
    grep "$id" rigid_sumary.log | awk '{printf "%s %s ",$1,$2;exit}' | tee -a sequences.txt
    getpdb.com $id | sequence.awk | awk '/Da chain:/{getline;printf "%s",$0}' | tee -a sequences.txt
    echo "" | tee -a sequences.txt
end



