#! /bin/tcsh -f
#
#	compute the reduced cell (P1) for each PDB entry
#
#
if(0) then
    # kill any runaway jobs
    ./tracer_killer.com
endif


append_file_date.com reduced_cells.txt
rm -f reduced_cells.txt

set cells = `cat CRYST1.txt | wc -l`
set line = 0
while ( $line <= $cells )
    @ line = ( $line + 1 )
    tail -n +$line CRYST1.txt | head -1 >! text.txt
    set id = `awk '{print $1}' text.txt`
    set CELL = `awk '{print $3,$4,$5,$6+0,$7+0,$8+0}' text.txt`
    set pdbSG = `awk '{print substr($0,56+5,10)}' text.txt`
    set latt  = `echo $pdbSG | awk '{print $1}'`
    set SG = `awk -v pdbSG="$pdbSG" -F "[\047]" 'pdbSG==$2{print;exit}' ${CLIBD}/symop.lib | awk '{print $4}'`
    set centering = `echo $pdbSG | awk '/^[ABCIF]/{c=substr($1,1,1);print c"CENTERED"}'`

    echo "$id $SG $pdbSG $CELL $centering"
    set test = `echo $latt $CELL | awk '{print ( /^[RH]/ && /90 90 120$/ )}'`
    if($test) then
	# get the rhombohedral cell - what you would get in indexing -
	set Hcell = ( $CELL )
 	echo "$Hcell" |\
	awk '{aH=$1;cH=$3;aR=sqrt(3*aH*aH+cH*cH)/3;\
	      sina2=3/2/sqrt(3+(cH/aH)^2);\
	      cosa2=sqrt(1-sina2^2);\
	      alphaR=atan2(sina2,cosa2)*2;\
	      alphaR=alphaR*90/atan2(1,0);\
	   print aR,aR,aR,alphaR,alphaR,alphaR}' |\
	cat >! Rcell.txt
	set CELL = `cat Rcell.txt`
    endif
    set tCELL = ( $CELL )

    set broken = 1
    set precision = 6
    while ( $broken )
        tracer << EOF >! tracer.log
$centering
CELL $tCELL
REDUCE
EOF
        set broken = `grep Lattice tracer.log | wc -l | awk '{print ( $1 < 2 )}'`
	if( $broken ) then
	    echo "precision = $precision"
	    set tCELL = `echo $CELL | awk -v precision=$precision 'BEGIN{srand();s=10^precision} {printf "%.12f %.12f %.12f %.12f %.12f %.12f", $1+rand()/s,$2+rand()/s,$3+rand()/s,$4+rand()/s,$5+rand()/s,$6+rand()/s}'`
	    echo "trying new cell: $tCELL"
	    set precision = `echo $precision | awk '{print $1*0.9}'`
	endif
    end

    cat tracer.log |\
    awk '/Lattice parameters of CELL/ && /conventional/{p=1}\
         p==1{print}\
         /Natural trigonometric/{p=0}\
    ' |\
    awk '/^ A /{A=$3;alpha=$6+0}\
         /^ B /{B=$3;beta=$6+0}\
         /^ C /{C=$3;gamma=$6+0}\
       END{print A,B,C,alpha,beta,gamma}' |\
    cat >! reduced_cell.txt

    set reduced_cell = `cat reduced_cell.txt`

    echo "$id $reduced_cell" | tee -a reduced_cells.txt

end




